#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   runGridEditor_server.py
#
#       gridEditor起動用（TreeFoamから起動する時に使用）
#         fieldのerrorCheck実施後これを起動する事。
#         ここでは、dialogが表示できないので、errorCheckを実施していない。
#
#       ＜使い方＞
#       runGridEditor.py <nTreeFoam> <title> <boundaryDir> <fieldDir>
#        nTreeFoam:     TreeFoamの起動No
#        title:         geridEditorのタイトル（case名、timeFolder名を含む）
#        boundaryDir:   polyMesh/baoundaryのdirectory
#        fieldDir:      timeFolderのdirectory
#
#   12/0919     新規作成
#      09/21    importするmoduleを整理
#   13/04/28    存在しないfieldがdisplayFieldsにあった場合、そのfieldを削除する。
#      05/01    全field表示設定の場合は、displayFieldsをrenameする。
#      07/11    boundaryDirを再取得（latest, start, firstTime, constantの順で探す）
#      10/13    多言語化対応
#   15/03/08    バグ修正。（表示順が正しく表示されなかったので、修正）
#      07/05    gridEditorの起動方法を修正。
#      11/28    gridEditorの起動方法を修正。（fieldの表示順の変更時も設定を保存。）
#   17/04/24    server用に修正
#

import os
import glob
import pyTreeFoam
import getFoamContents
from sys import argv

serverName = ""

#import logFileCreater; logFileCreater.log()

#
# getFields
#   編集するfieldを取得
def getFields(fieldDir):
    fileName = pyTreeFoam.getSaveDisplayFieldsFileName(fieldDir)
    if len(glob.glob(fileName)) == 0:
        fields = pyTreeFoam.getFieldNames(fieldDir)
        fields.sort()
    else:
        fieldsD = pyTreeFoam.getDisplayFields(fileName)
        fieldsC = pyTreeFoam.getFieldNames(fieldDir)
        fields = []
        for field in fieldsD:
            for name in fieldsC:
                if field == name:
                    fields.append(field)
                    break
    return fields

#
#  createTempCase
#    tempCaseDirを作成
def createTempCase(boundaryDir, fieldDir):
    #serverName = pyTreeFoam.getServerName(boundaryDir)
    data = pyTreeFoam.readSshfsData(serverName)
    hostName = data[0]
    userName = data[1]
    hostDir = data[2]
    mountDir = data[3]

    #caseDirを取得
    words = fieldDir.split("/")
    caseName = ""
    try:
        a = float(words[-1])
        caseDir = "/".join(words[:-1])
        caseName = words[-2]
        numFolder = words[-1]
    except:
        pass
    if caseName == "":
        try:
            #multiRegionの場合
            a = float(words[-2])
            caseDir = "/".join(words[:-2])
            caseName = words[-3]
            numFolder = words[-2]
        except:
            #その他folder
            caseDir = "/".join(words[:-1])
            caseName = words[-2]
            numFolder = words[-1]
            if numFolder == "":
                numFolder = caseName
    tempCaseDir = os.getenv("TreeFoamUserPath") + "/temp/server_" + caseName + "/" + caseName
    #serverにtempCaseを作成
    serverCaseDir = hostDir + caseDir[len(mountDir):]
    resultFile = "output"
    comm = "python ~/TreeFoamServer/bin/server_createCaseTemp.py " + serverCaseDir + " " + numFolder
    stat = pyTreeFoam.getTreeFoamServerResult(serverName, comm, resultFile)
    if stat != "OK":
        return [caseDir, tempCaseDir]
    #  tempFolder名を取得
    resFile = os.getenv("TreeFoamUserPath") + "/temp/" + resultFile
    f=open(resFile); cont=f.read(); f.close()
    tempFolder = cont.split()[0]
    #  tempCaseDirを取得
    tempCaseDir = os.getenv("TreeFoamUserPath") + "/temp/" + tempFolder + "/" + caseName

    #localにtempCaseを作成
    #  tempCaseDirのチェック
    if len(glob.glob(tempCaseDir)) == 0:
        #全てコピー
        sourceFolder = "~/TreeFoamServer/tmp/" + tempFolder
        toCopyFolder = os.getenv("TreeFoamUserPath") + "/temp"
    else:
        #timeFolderのみコピー
        sourceFolder = "~/TreeFoamServer/tmp/" + tempFolder + "/" + caseName + "/" + numFolder
        toCopyFolder = os.getenv("TreeFoamUserPath") + "/temp/" + tempFolder + "/" + caseName
    #  caseをlocalに転送
    [stat, cont] = pyTreeFoam.scpCommand(sourceFolder, toCopyFolder, serverName)
    #  serverNameの書き込み
    fileName = os.getenv("TreeFoamUserPath") + "/temp/" + tempFolder + "/serverName"
    f = open(fileName, "w"); f.write(serverName.encode("utf-8")); f.close()
    return [caseDir, tempCaseDir]


#
#  deleteTempCaseDir
#    tempCaseDirを削除
def deleteTempCaseDir():
    currTimeDir = os.path.abspath(fieldDir)
    #他のgridEditorのeditDieをチェック
    fileName = os.getenv("TreeFoamUserPath") + "/data/nRunningGrid"
    f=open(fileName); lines=f.readlines(); f.close()
    gridList = []
    for line in lines:
        words = line.split()
        if len(words) > 0:
            gridList.append(words)
    flag = 0
    for gridCont in gridList:
        timeDir = os.path.abspath(gridCont[2])
        if timeDir == currTimeDir:
            flag = 1
            break
    if flag == 1:
        return

    #tempCaseDirを削除
    #  caseDirとtimeFolderを取得
    words = fieldDir.split("/")
    try:
        a = float(words[-1])
        timeFolder = words[-1]
        caseDir = "/".join(words[:-1])
    except:
        timeFolder = words[-2]
        caseDir = "/".join(words[:-2])
    timeFolders = pyTreeFoam.getTimeFolders(caseDir)
    if len(timeFolders) == 1:
        #tempCaseを削除
        tempDir = "/".join(caseDir.split("/")[:-1])
        comm = "rm -rf " + tempDir
        os.system(comm)
        tempFolder = tempDir.split("/")[-1]
        serverTempDir = "~/TreeFoamServer/tmp/" + tempFolder
        comm = "rm -rf " + serverTempDir
        pyTreeFoam.sendSshCommand(comm, serverName)
    else:
        #timeFolderのみ削除
        timeDir = caseDir + "/" + timeFolder
        comm = "rm -rf " + timeDir
        os.system(comm)
        tempDir = "/".join(caseDir.split("/")[:-1])
        caseName = caseDir.split("/")[-1]
        tempFolder = tempDir.split("/")[-1]
        serverTimeDir = "~/TreeFoamServer/tmp/" + tempFolder + "/" + caseName + "/" + timeFolder
        comm = "rm -rf " + serverTimeDir
        pyTreeFoam.sendSshCommand(comm, serverName)
    return
    
#
#  saveServer
#    server内に保存
def saveServer(caseDir):
    tempFolder = "/".join(caseDir.split("/")[:-1])
    #serverCaseDirを取得
    fileName = tempFolder + "/caseDir"
    f=open(fileName); cont=f.read(); f.close()
    serverCaseDir = cont.split()[0]
    #serverNameを取得
    fileName = tempFolder + "/serverName"
    f=open(fileName); cont=f.read(); f.close()
    serverName = cont.split()[0]
    #scpでserverに保存
    source = caseDir
    pasteDir = "/".join(serverCaseDir.split("/")[:-1])
    [stat, cont] = pyTreeFoam.scpCommand(source, pasteDir, serverName)
    return [stat, cont]


if __name__ == "__main__":
    #言語の設定
    import gettext
    treeFoamPath = os.environ["TreeFoamPath"]
    gettext.install("treefoam", treeFoamPath + "/data/locale")

    #引数を取得
    nTreeFoam   = argv[1]
    title       = argv[2]
    boundaryDir = argv[3]
    fieldDir    = argv[4]

    #tempCaseを作成
    serverName = pyTreeFoam.getServerName(boundaryDir)
    [caseDir, tempCaseDir] = createTempCase(boundaryDir, fieldDir)

    #boundaryとfieldDirを書き換え
    boundaryDir = tempCaseDir + boundaryDir[len(caseDir):]
    fieldDir = tempCaseDir + fieldDir[len(caseDir):]

    #fieldを取得
    fields = getFields(fieldDir)

    #boundaryDirを取得
    folder = fieldDir.split("/")[-1]
    try:
        a=float(folder)
        currTime = folder
        caseDir = "/".join(fieldDir.split("/")[:-1])
        region = "."
    except:
        currTime = fieldDir.split("/")[-2]
        caseDir = "/".join(fieldDir.split("/")[:-2])
        region = folder
    relativeDir = getFoamContents.getCurrMeshDir(caseDir, currTime, region, "boundary")
    boundaryDir = caseDir + "/" + relativeDir

    #gridEditorを起動
    var = nTreeFoam + " '" + title + "' " + " ".join([boundaryDir] + [fieldDir] + fields)
    comm = "editBoundaryConditionsDialog_server.py " + var
    os.system(comm)

    #tempCaseDirを削除
    deleteTempCaseDir()


